/*******************************************************************************
 * Copyright (c) 2005 Actuate Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Actuate Corporation  - initial API and implementation
 *******************************************************************************/

package org.eclipse.birt.report.designer.data.ui.dataset;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.birt.report.designer.data.ui.property.AbstractDescriptionPropertyPage;
import org.eclipse.birt.report.designer.data.ui.util.ControlProvider;
import org.eclipse.birt.report.designer.data.ui.util.DataSetExpressionProvider;
import org.eclipse.birt.report.designer.data.ui.util.Utility;
import org.eclipse.birt.report.designer.internal.ui.util.ExceptionHandler;
import org.eclipse.birt.report.designer.nls.Messages;
import org.eclipse.birt.report.designer.ui.ReportPlugin;
import org.eclipse.birt.report.designer.ui.dialogs.ExpressionBuilder;
import org.eclipse.birt.report.designer.util.DEUtil;
import org.eclipse.birt.report.model.api.AggregationArgumentHandle;
import org.eclipse.birt.report.model.api.ColumnHintHandle;
import org.eclipse.birt.report.model.api.ComputedColumnHandle;
import org.eclipse.birt.report.model.api.DataSetHandle;
import org.eclipse.birt.report.model.api.DesignElementHandle;
import org.eclipse.birt.report.model.api.PropertyHandle;
import org.eclipse.birt.report.model.api.StructureFactory;
import org.eclipse.birt.report.model.api.activity.SemanticException;
import org.eclipse.birt.report.model.api.elements.DesignChoiceConstants;
import org.eclipse.birt.report.model.api.elements.structures.AggregationArgument;
import org.eclipse.birt.report.model.api.elements.structures.ComputedColumn;
import org.eclipse.birt.report.model.api.metadata.IArgumentInfo;
import org.eclipse.birt.report.model.api.metadata.IArgumentInfoList;
import org.eclipse.birt.report.model.api.metadata.IChoice;
import org.eclipse.birt.report.model.api.metadata.IMethodInfo;
import org.eclipse.birt.report.model.core.Structure;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.IMessageProvider;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.Text;

/**
 * TODO: Please document
 * 
 * @version $Revision: 1.19 $ $Date: 2007/06/08 11:49:11 $
 */
public class DataSetComputedColumnsPage extends AbstractDescriptionPropertyPage
		implements
			ITableLabelProvider
{

	private transient PropertyHandle computedColumns = null;
	private transient PropertyHandleTableViewer viewer = null;
	private static String[] cellLabels = new String[]{
			Messages.getString( "dataset.editor.title.columnName" ),//$NON-NLS-1$
			Messages.getString( "dataset.editor.title.dataType" ),//$NON-NLS-1$
			Messages.getString( "dataset.editor.title.expression" ),//$NON-NLS-1$
			Messages.getString( "dataset.editor.title.aggrFunc" ),
			Messages.getString( "dataset.editor.title.aggrArgu" ),
			Messages.getString( "dataset.editor.title.filter" )
	};

	private static String[] cellProperties = new String[]{
			ComputedColumn.NAME_MEMBER,
			ComputedColumn.DATA_TYPE_MEMBER,
			ComputedColumn.EXPRESSION_MEMBER,
			ComputedColumn.AGGREGATEON_FUNCTION_MEMBER,
			ComputedColumn.ARGUMENTS_MEMBER,
			ComputedColumn.FILTER_MEMBER
	};

	private static IChoice[] dataTypes = DEUtil.getMetaDataDictionary( )
			.getStructure( ComputedColumn.COMPUTED_COLUMN_STRUCT )
			.getMember( ComputedColumn.DATA_TYPE_MEMBER )
			.getAllowedChoices( )
			.getChoices( );

	private static List aggrFuncs = DEUtil.getMetaDataDictionary( )
			.getFunctions( );

	public static String[] funcNames;

	public static Map funcArgMap = new HashMap( );

	static
	{
		funcNames = new String[aggrFuncs.size( )];
		int index = 0;
		for ( Iterator i = aggrFuncs.iterator( ); i.hasNext( ); index++ )
		{
			IMethodInfo function = (IMethodInfo) i.next( );
			funcNames[index] = function.getName( );
			Iterator argListItr = function.argumentListIterator( );
			IArgumentInfoList arguments = (IArgumentInfoList) argListItr.next( );
			for ( Iterator j = arguments.argumentsIterator( ); j.hasNext( ); )
			{
				IArgumentInfo argInfo = (IArgumentInfo) j.next( );
				funcArgMap.put( function.getName( ), argInfo.getName( ) );
			}
		}
	}

	/**
	 * 
	 */
	public DataSetComputedColumnsPage( )
	{
		super( );
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.birt.report.designer.ui.dialogs.properties.IPropertyPage#createPageControl(org.eclipse.swt.widgets.Composite)
	 */
	public Control createContents( Composite parent )
	{
		computedColumns = ( (DataSetHandle) getContainer( ).getModel( ) ).getPropertyHandle( DataSetHandle.COMPUTED_COLUMNS_PROP );;
		viewer = new PropertyHandleTableViewer( parent, true, true, true );
		TableColumn column = new TableColumn( viewer.getViewer( ).getTable( ),
				SWT.LEFT );
		column.setText( cellLabels[0] );
		column.setWidth( 150 );
		column = new TableColumn( viewer.getViewer( ).getTable( ), SWT.LEFT );
		column.setText( cellLabels[1] );
		column.setWidth( 200 );
		column = new TableColumn( viewer.getViewer( ).getTable( ), SWT.LEFT );
		column.setText( cellLabels[2] );
		column.setWidth( 200 );
		column = new TableColumn( viewer.getViewer( ).getTable( ), SWT.LEFT );
		column.setText( cellLabels[3] );
		column.setWidth( 200 );
		viewer.getViewer( )
				.setContentProvider( new IStructuredContentProvider( ) {

					public Object[] getElements( Object inputElement )
					{
						ArrayList computedColumnsList = new ArrayList( 10 );
						Iterator iter = computedColumns.iterator( );
						if ( iter != null )
						{
							while ( iter.hasNext( ) )
							{
								computedColumnsList.add( iter.next( ) );
							}
						}
						return computedColumnsList.toArray( );
					}

					public void dispose( )
					{

					}

					public void inputChanged( Viewer viewer, Object oldInput,
							Object newInput )
					{

					}
				} );
		viewer.getViewer( ).setLabelProvider( this );
		viewer.getViewer( ).setInput( computedColumns );
		addListeners( );
		setToolTips( );
		return viewer.getControl( );
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.birt.report.designer.ui.dialogs.properties.IPropertyPage#pageActivated()
	 */
	public void pageActivated( )
	{
		refreshColumnNames( );
		getContainer( ).setMessage( Messages.getString( "dataset.editor.computedColumns" ),
				IMessageProvider.NONE ); // $-NON-NLS-1
		// //$NON-NLS-1$
		setPageProperties( );

		computedColumns = ( (DataSetHandle) getContainer( ).getModel( ) ).getPropertyHandle( DataSetHandle.COMPUTED_COLUMNS_PROP );;
		viewer.getViewer( ).setInput( computedColumns );
		viewer.getViewer( ).getTable( ).select( 0 );
	}

	/**
	 * Refresh columns meta data
	 */
	private void refreshColumnNames( )
	{
		( (DataSetEditor) this.getContainer( ) ).getCurrentItemModel( true,
				true );
	}

	private void addListeners( )
	{
		viewer.getNewButton( ).addSelectionListener( new SelectionAdapter( ) {

			public void widgetSelected( SelectionEvent e )
			{
				doNew( );
			}
		} );

		viewer.getEditButton( ).addSelectionListener( new SelectionAdapter( ) {

			public void widgetSelected( SelectionEvent e )
			{
				doEdit( );
			}
		} );

		viewer.getViewer( ).getTable( ).addMouseListener( new MouseAdapter( ) {

			public void mouseDoubleClick( MouseEvent e )
			{
				doEdit( );
			}
		} );

		viewer.getViewer( ).getTable( ).addKeyListener( new KeyListener( ) {

			public void keyPressed( KeyEvent e )
			{
			}

			public void keyReleased( KeyEvent e )
			{
				if ( e.keyCode == SWT.DEL )
				{
					setPageProperties( );
				}
			}

		} );

		viewer.getRemoveButton( )
				.addSelectionListener( new SelectionListener( ) {

					public void widgetSelected( SelectionEvent e )
					{
						setPageProperties( );
					}

					public void widgetDefaultSelected( SelectionEvent e )
					{
					}

				} );

		viewer.getRemoveMenuItem( )
				.addSelectionListener( new SelectionListener( ) {

					public void widgetSelected( SelectionEvent e )
					{
						setPageProperties( );
					}

					public void widgetDefaultSelected( SelectionEvent e )
					{
					}

				} );

		viewer.getRemoveAllMenuItem( )
				.addSelectionListener( new SelectionListener( ) {

					public void widgetSelected( SelectionEvent e )
					{
						setPageProperties( );
					}

					public void widgetDefaultSelected( SelectionEvent e )
					{
						widgetSelected( e );
					}
				} );

		viewer.getViewer( )
				.addSelectionChangedListener( new ViewerSelectionListener( ) );
	}

	private void doNew( )
	{
		doEdit( new ComputedColumn( ) );
	}

	private void doEdit( )
	{
		int index = viewer.getViewer( ).getTable( ).getSelectionIndex( );
		if ( index == -1 )
			return;

		ComputedColumnHandle handle = (ComputedColumnHandle) viewer.getViewer( )
				.getTable( )
				.getItem( index )
				.getData( );

		doEdit( handle );
	}

	private void doEdit( Object structureOrHandle )
	{
		ComputedColumnInputDialog dlg = new ComputedColumnInputDialog( structureOrHandle );

		if ( dlg.open( ) == Window.OK )
		{
			update( structureOrHandle );
		}
	}
	
	private void update( Object structureOrHandle )
	{
		if ( structureOrHandle instanceof ComputedColumn )
		{
			try
			{
				computedColumns.addItem( (ComputedColumn) structureOrHandle );
				viewer.getViewer( ).refresh( );
			}
			catch ( SemanticException e )
			{
				ExceptionHandler.handle( e );
			}
		}
		else
		{
			viewer.getViewer( ).update( structureOrHandle, null );
		}

		if ( validateAllComputedColumns( ) )
			getContainer( ).setMessage( Messages.getString( "dataset.editor.computedColumns" ),
					IMessageProvider.NONE );
	}
	
	private ComputedColumn getStructure( Object structureOrHandle )
	{
		ComputedColumn structure = null;
		if ( structureOrHandle instanceof ComputedColumn )
		{
			structure = (ComputedColumn) structureOrHandle;
		}
		else
		{
			structure = (ComputedColumn) ( (ComputedColumnHandle) structureOrHandle ).getStructure( );
		}

		return structure;
	}
	
	private final String getTypeName( String typeDisplayName )
	{
		for ( int i = 0; i < dataTypes.length; i++ )
		{
			if ( dataTypes[i].getDisplayName( ).equals( typeDisplayName ) )
				return dataTypes[i].getName( );
		}

		return dataTypes[0].getName( );
	}

	private final String getTypeDisplayName( String typeName )
	{
		for ( int i = 0; i < dataTypes.length; i++ )
		{
			if ( dataTypes[i].getName( ).equals( typeName ) )
			{
				return dataTypes[i].getDisplayName( );
			}
		}

		return dataTypes[0].getDisplayName( );
	}
    
    private void setToolTips( )
	{
		viewer.getNewButton( )
				.setToolTipText( Messages.getString( "DataSetComputedColumnsPage.toolTipText.New" ) );//$NON-NLS-1$
		viewer.getEditButton( )
				.setToolTipText( Messages.getString( "DataSetComputedColumnsPage.toolTipText.Edit" ) );//$NON-NLS-1$
		viewer.getRemoveButton( )
				.setToolTipText( Messages.getString( "DataSetComputedColumnsPage.toolTipText.Remove" ) );//$NON-NLS-1$
		viewer.getUpButton( )
				.setToolTipText( Messages.getString( "DataSetComputedColumnsPage.toolTipText.Up" ) );//$NON-NLS-1$
		viewer.getDownButton( )
				.setToolTipText( Messages.getString( "DataSetComputedColumnsPage.toolTipText.Down" ) );//$NON-NLS-1$
	}
    
    /*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnImage(java.lang.Object,
	 *      int)
	 */
	public Image getColumnImage( Object element, int columnIndex )
	{
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnText(java.lang.Object,
	 *      int)
	 */
	public String getColumnText( Object element, int columnIndex )
	{
		String value = null;
		ComputedColumn computedColumn = getStructure( element );

		try
		{
			switch ( columnIndex )
			{
				case 0 :
				{
					value = computedColumn.getName( );
					break;
				}
				case 1 :
				{
					value = getTypeDisplayName( computedColumn.getDataType( ) );
					break;
				}
				case 2 :
				{
					value = computedColumn.getExpression( );
					break;
				}
				case 3 :
				{
					value = computedColumn.getAggregateFunction( );
					for( int i = 0; i < aggrFuncs.size( ); i++ )
					{
						if( ((IMethodInfo)aggrFuncs.get( i )).getName( ).equals( value ))
						{
							value = ((IMethodInfo)aggrFuncs.get( i )).getDisplayName( );
							break;
						}
					}
					
					break;
				}
			}
		}
		catch ( Exception e )
		{
			ExceptionHandler.handle( e );
		}
		
		return ( value == null ? "" : value ); //$NON-NLS-1$
	}

    /*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.IBaseLabelProvider#addListener(org.eclipse.jface.viewers.ILabelProviderListener)
	 */
	public void addListener( ILabelProviderListener listener )
	{
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.IBaseLabelProvider#dispose()
	 */
	public void dispose( )
	{
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.IBaseLabelProvider#isLabelProperty(java.lang.Object,
	 *      java.lang.String)
	 */
	public boolean isLabelProperty( Object element, String property )
	{
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.IBaseLabelProvider#removeListener(org.eclipse.jface.viewers.ILabelProviderListener)
	 */
	public void removeListener( ILabelProviderListener listener )
	{
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.birt.report.designer.ui.dialogs.properties.AbstractDescriptionPropertyPage#getPageDescription()
	 */
	public String getPageDescription( )
	{
		return Messages.getString( "DataSetComputedColumnsPage.description" ); //$NON-NLS-1$
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.birt.report.designer.ui.dialogs.properties.AbstractPropertyPage#performOk()
	 */
	public boolean performOk( )
	{
		if ( validateAllComputedColumns( ) )
		{
			return super.performOk( );
		}
		return false;
	}

	private class ViewerSelectionListener implements ISelectionChangedListener
	{

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.ISelectionChangedListener#selectionChanged(org.eclipse.jface.viewers.SelectionChangedEvent)
		 */
		public void selectionChanged( SelectionChangedEvent event )
		{
			// TODO Auto-generated method stub
			setPageProperties( );
		}

	}

	/**
	 * Depending on the Computed the properties of various controls on this page
	 * are set
	 */
	private void setPageProperties( )
	{
		boolean computedColumnsExist = false;

		computedColumnsExist = ( computedColumns != null
				&& computedColumns.getListValue( ) != null && computedColumns.getListValue( )
				.size( ) > 0 );
		viewer.getEditButton( ).setEnabled( computedColumnsExist );
		viewer.getDownButton( ).setEnabled( computedColumnsExist
				&& computedColumns.getListValue( ).size( ) > 1 );
		viewer.getUpButton( ).setEnabled( computedColumnsExist
				&& computedColumns.getListValue( ).size( ) > 1 );
		viewer.getRemoveButton( ).setEnabled( computedColumnsExist );
		viewer.getRemoveMenuItem( ).setEnabled( computedColumnsExist );
		viewer.getRemoveAllMenuItem( ).setEnabled( computedColumnsExist );
		validateAllComputedColumns( );
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.birt.report.designer.ui.dialogs.properties.IPropertyPage#getToolTip()
	 */
	public String getToolTip( )
	{
		return Messages.getString( "DataSetComputedColumnsPage.ComputedColumns.Tooltip" ); //$NON-NLS-1$
	}

	public boolean canLeave( )
	{
		return validateAllComputedColumns( );
	}

	/*
	 * Exam whether all registered computed columns are valid. Set the message
	 * of dialog as appropriated
	 */
	private boolean validateAllComputedColumns( )
	{
		if ( computedColumns != null )
		{
			Iterator iter = computedColumns.iterator( );
			if ( iter != null )
			{
				while ( iter.hasNext( ) )
				{
					Object o = iter.next( );
					ComputedColumn computedColumn = null;
					if ( o instanceof ComputedColumnHandle )
					{
						computedColumn = (ComputedColumn) ( (ComputedColumnHandle) o ).getStructure( );
					}
					else
					{
						computedColumn = (ComputedColumn) o;
					}
					if ( !validateSingleColumn( computedColumn ) )
						return false;
				}
			}
		}
		return true;
	}

	/*
	 * Exam whether one computed column is valid or not.
	 */
	private boolean validateSingleColumn( ComputedColumn computedColumn )
	{
		if ( computedColumn.getName( ) == null
				|| computedColumn.getName( ).trim( ).length( ) == 0 )
		{
			getContainer( ).setMessage( Messages.getFormattedString( "dataset.editor.error.missingComputedColumnName", new Object[]{computedColumn.getName( )} ), IMessageProvider.ERROR ); //$NON-NLS-1$
			return false;
		}
		if ( computedColumn.getExpression( ) == null
				|| computedColumn.getExpression( ).trim( ).length( ) == 0 )
		{
			if ( isFunctionCount( computedColumn.getAggregateFunction( ) ) == false )
			{
				getContainer( ).setMessage( Messages.getFormattedString( "dataset.editor.error.missingComputedColumnExpression", new Object[]{computedColumn.getName( )} ), IMessageProvider.ERROR ); //$NON-NLS-1$
				return false;
			}
		}

		Iterator iter = ( (DataSetHandle) getContainer( ).getModel( ) ).getPropertyHandle( DataSetHandle.COLUMN_HINTS_PROP )
				.iterator( );
		while ( iter.hasNext( ) )
		{
			ColumnHintHandle hint = (ColumnHintHandle) iter.next( );
			if ( computedColumn.getName( ).equals( hint.getAlias( ) )
					|| computedColumn.getName( ).equals( hint.getColumnName( ) ) )
			{
				getContainer( ).setMessage( Messages.getFormattedString( "dataset.editor.error.computedColumnNameAlreadyUsed", new Object[]{computedColumn.getName( )} ), IMessageProvider.ERROR ); //$NON-NLS-1$
				return false;
			}
		}
		return true;
	}

	/**
	 * @param funcName
	 */
	private boolean isFunctionCount( String funcName )
	{
		return DesignChoiceConstants.AGGREGATION_FUNCTION_COUNT.equals( funcName )
				|| DesignChoiceConstants.AGGREGATION_FUNCTION_RUNNINGCOUNT.equals( funcName );
	}

	private class ComputedColumnInputDialog extends PropertyHandleInputDialog
	{

		private Text columnName = null;
		private Combo dataType = null;
		private Combo aggrCombo = null;
		private Text expression = null;
		private Text argument = null;
		private Label argumentLabel = null;
		private Text filter = null;
		private Button argumentButton;

		private ComputedColumnInputDialog( Object structureOrHandle )
		{
			super( structureOrHandle );
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.birt.report.designer.data.ui.dataset.PropertyHandleInputDialog#createCustomControls(org.eclipse.swt.widgets.Composite)
		 */
		protected void createCustomControls( Composite parent )
		{
			createTextCell( parent, 0 );
			createComboBoxCell( parent, 1 );
			createExpressionCell( parent, 2 );
			createAggrListCell( parent, 3 );
			createArgumentCell( parent, 4 );
			createFilterCell( parent, 5 );
		}

		private void createTextCell( Composite parent, final int index )
		{
			ControlProvider.createLabel( parent, cellLabels[index] );

			columnName = ControlProvider.createText( parent,
					(String) getProperty( getStructureOrHandle( ),
							cellProperties[index] ) );
			columnName.setLayoutData( ControlProvider.getGridDataWithHSpan( 2 ) );
			columnName.addModifyListener( new ModifyListener( ) {

				public void modifyText( ModifyEvent e )
				{
					validateSyntax( );
				}

			} );
		}

		private void createComboBoxCell( Composite parent, final int index )
		{
			ControlProvider.createLabel( parent, cellLabels[index] );

			dataType = ControlProvider.createCombo( parent, SWT.READ_ONLY );
			dataType.setLayoutData( ControlProvider.getGridDataWithHSpan( 2 ) );
			dataType.setItems( getDisplayNames( dataTypes ) );
			dataType.select( Utility.findIndex( dataType.getItems( ),
					getTypeDisplayName( (String) getProperty( getStructureOrHandle( ),
							cellProperties[index] ) ) ) );
			dataType.addSelectionListener( new SelectionAdapter( ) {

				public void widgetSelected( SelectionEvent e )
				{
					validateSyntax( );
				}

			} );
		}

		private void createAggrListCell( Composite parent, final int index )
		{
			ControlProvider.createLabel( parent, cellLabels[index] );

			aggrCombo = ControlProvider.createCombo( parent, SWT.READ_ONLY );
			aggrCombo.setLayoutData( ControlProvider.getGridDataWithHSpan( 2 ) );
			String[] aggrFunctionNames = getFuncDisplayNames( );
			aggrCombo.add( "" );
			for ( int i = 0; i < aggrFunctionNames.length; i++ )
				aggrCombo.add( aggrFunctionNames[i] );

			String aggrFuncName = (String) getProperty( getStructureOrHandle( ),
					cellProperties[index] );
			int index1 = 0;
			for ( int i = 1; i <= aggrFuncs.size( ); i++ )
			{
				if ( ( (IMethodInfo) aggrFuncs.get( i - 1 ) ).getName( )
						.equals( aggrFuncName ) )
				{
					index1 = i;
				}
			}

			aggrCombo.select( index1 );
			aggrCombo.addSelectionListener( new SelectionAdapter( ) {

				public void widgetSelected( SelectionEvent e )
				{
					validateSyntax( );
					updateArgumentUIStatus( );
				}

			} );
		}

		/**
		 * 
		 */
		private void updateArgumentUIStatus( )
		{
			if ( needArguments( ) )
			{
				argument.setEnabled( true );
				argumentLabel.setEnabled( true );
				argumentButton.setEnabled( true );
			}
			else
			{
				argument.setEnabled( false );
				argumentLabel.setEnabled( false );
				argumentButton.setEnabled( false );
				argument.setText( "" );
			}
		}

		/**
		 * @return
		 */
		private boolean needArguments( )
		{
			String selectedFuncName = getSelectedFuncName( aggrCombo );
			if ( selectedFuncName == null )
				return false;
			String funcArgName = getFunctionArgName( selectedFuncName );
			return funcArgName != null;
		}

		private void createArgumentCell( Composite parent, final int index )
		{
			argumentLabel = ControlProvider.createLabel( parent,
					cellLabels[index] );

			Composite composite = ControlProvider.getDefaultComposite( parent );
			Object handle = this.getStructureOrHandle( );
			if ( handle instanceof ComputedColumnHandle )
			{
				ComputedColumnHandle cHandle = (ComputedColumnHandle) handle;
				AggregationArgumentHandle argHandle = (AggregationArgumentHandle) cHandle.argumentsIterator( )
						.next( );
				if ( argHandle == null )
					argument = ControlProvider.createText( composite, null );
				else
					argument = ControlProvider.createText( composite,
							argHandle.getValue( ) );

			}
			else
				argument = ControlProvider.createText( composite,
						(String) getProperty( getStructureOrHandle( ),
								cellProperties[index] ) );
			argument.setLayoutData( ControlProvider.getGridDataWithHSpan( 1 ) );
			argument.addModifyListener( new ModifyListener( ) {

				public void modifyText( ModifyEvent e )
				{
					validateSyntax( );
				}

			} );

			SelectionAdapter listener = new SelectionAdapter( ) {

				public void widgetSelected( SelectionEvent event )
				{
					ExpressionBuilder expressionBuilder = new ExpressionBuilder( argument.getText( ) );
					expressionBuilder.setExpressionProvier( new DataSetExpressionProvider( (DesignElementHandle) getContainer( ).getModel( ) ) );

					if ( expressionBuilder.open( ) == OK )
					{
						argument.setText( expressionBuilder.getResult( ).trim( ) );
					}
				}
			};

			argumentButton = ControlProvider.createButton( composite,
					SWT.PUSH,
					listener );
			updateArgumentUIStatus( );
		}

		private void createFilterCell( Composite parent, final int index )
		{
			ControlProvider.createLabel( parent, cellLabels[index] );
 
			Composite composite = ControlProvider.getDefaultComposite( parent );
			Object handle = this.getStructureOrHandle( );
			if ( handle instanceof ComputedColumnHandle )
			{
				ComputedColumnHandle cHandle = (ComputedColumnHandle) handle;
				filter = ControlProvider.createText( composite,
						cHandle.getFilterExpression( ) );

			}
			else
				filter = ControlProvider.createText( composite,
						(String) getProperty( getStructureOrHandle( ),
								cellProperties[index] ) );
			filter.setLayoutData( ControlProvider.getGridDataWithHSpan( 1 ) );
			filter.addModifyListener( new ModifyListener( ) {

				public void modifyText( ModifyEvent e )
				{
					validateSyntax( );
				}

			} );

			SelectionAdapter listener = new SelectionAdapter( ) {

				public void widgetSelected( SelectionEvent event )
				{
					ExpressionBuilder expressionBuilder = new ExpressionBuilder( filter.getText( ) );
					expressionBuilder.setExpressionProvier( new DataSetExpressionProvider( (DesignElementHandle) getContainer( ).getModel( ) ) );

					if ( expressionBuilder.open( ) == OK )
					{
						filter.setText( expressionBuilder.getResult( ).trim( ) );
					}
				}
			};

			ControlProvider.createButton( composite, SWT.PUSH, listener );
		}

		private String[] getDisplayNames( IChoice[] choice )
		{
			String[] dataTypeDisplayNames = new String[choice.length];
			for ( int i = 0; i < choice.length; i++ )
			{
				dataTypeDisplayNames[i] = choice[i].getDisplayName( );
			}

			return dataTypeDisplayNames;

		}

		private String[] getFuncDisplayNames( )
		{
			String[] result = new String[aggrFuncs.size( )];
			for ( int i = 0; i < aggrFuncs.size( ); i++ )
			{
				result[i] = ( (IMethodInfo) aggrFuncs.get( i ) ).getDisplayName( );
			}
			return result;
		}

		/**
		 * The aggrCombo contains the display names of all functions and "".
		 * 
		 * @param aggrCombo
		 * @return
		 */
		private String getSelectedFuncName( Combo aggrCombo )
		{
			int index = aggrCombo.getSelectionIndex( );
			if ( index == 0 )
				return null;
			return funcNames[index - 1];
		}

		private void createExpressionCell( Composite parent, final int index )
		{
			ControlProvider.createLabel( parent, cellLabels[index] );

			Composite composite = ControlProvider.getDefaultComposite( parent );
			expression = ControlProvider.createText( composite,
					(String) getProperty( getStructureOrHandle( ),
							cellProperties[index] ) );
			expression.setLayoutData( ControlProvider.getGridDataWithHSpan( 1 ) );
			expression.addModifyListener( new ModifyListener( ) {

				public void modifyText( ModifyEvent e )
				{
					validateSyntax( );
				}

			} );

			SelectionAdapter listener = new SelectionAdapter( ) {

				public void widgetSelected( SelectionEvent event )
				{
					ExpressionBuilder expressionBuilder = new ExpressionBuilder( expression.getText( ) );
					expressionBuilder.setExpressionProvier( new DataSetExpressionProvider( (DesignElementHandle) getContainer( ).getModel( ) ) );

					if ( expressionBuilder.open( ) == OK )
					{
						expression.setText( expressionBuilder.getResult( ).trim( ) );
					}
				}
			};

			ControlProvider.createButton( composite, SWT.PUSH, listener );
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.birt.report.designer.data.ui.dataset.PropertyHandleInputDialog#validateSemantics(java.lang.Object)
		 */
		protected IStatus validateSemantics( Object structureOrHandle )
		{
			AggregationArgument aggrArgument = null;
			String cName = columnName.getText( );
			String dataTypeName = getTypeName( dataType.getText( ) );
			String expr = expression.getText( );
			String funcName = getSelectedFuncName( this.aggrCombo );
			String argValue = this.argument.getText( );
			String filter = this.filter.getText( );
			
			try
			{
				if ( structureOrHandle instanceof ComputedColumnHandle )
				{
					ComputedColumnHandle handle = (ComputedColumnHandle) structureOrHandle;
					handle.setName( cName );
					handle.setDataType( dataTypeName );
					handle.setExpression( expr );
					handle.setAggregateFunction( funcName );
					handle.setFilterExpression( filter );
					handle.clearArgumentList( );
					if ( argValue != null && argValue.trim( ).length( ) > 0 )
					{
						String argName = getFunctionArgName( funcName );
						if ( argName != null )
						{
							aggrArgument = StructureFactory.createAggregationArgument( );
							aggrArgument.setName( argName );
							handle.addArgument( aggrArgument );
							aggrArgument.setValue( argValue );
						}
					}
				}
				else if ( structureOrHandle instanceof ComputedColumn )
				{

					ComputedColumn handle = (ComputedColumn) structureOrHandle;
					handle.setName( cName );
					handle.setDataType( dataTypeName );
					handle.setExpression( expr );
					handle.setAggregateFunction( funcName );
					handle.setFilterExpression( filter );
					if ( argValue != null && argValue.trim( ).length( ) > 0 )
					{
						String argName = getFunctionArgName( funcName );
						if ( argName != null )
						{
							aggrArgument = StructureFactory.createAggregationArgument( );
							aggrArgument.setName( argName );
							handle.addArgument( aggrArgument );
							aggrArgument.setValue( argValue );
						}
					}
				}
			}
			catch ( Exception e )
			{
				Status status = new Status( IStatus.ERROR,
						ReportPlugin.REPORT_UI,
						"",
						e );
				return status;
			}
			return getOKStatus( );
		}

		/**
		 * 
		 * @param funcName
		 * @return
		 */
		private String getFunctionArgName( String funcName )
		{
			return (String) funcArgMap.get( funcName );
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.birt.report.designer.data.ui.dataset.PropertyHandleInputDialog#validateSyntax(java.lang.Object)
		 */
		protected IStatus validateSyntax( Object structureOrHandle )
		{
			// duplicated columnName check
			if ( !isUniqueColumnName( ) )
				return getMiscStatus( IStatus.ERROR,
						Messages.getString( "DataSetComputedColumnsPage.duplicatedName" ) );

			// blankProperty check
			if ( isBlankProperty( columnName.getText( ) ) )
				return getBlankPropertyStatus( cellLabels[0] );
			if ( isBlankProperty( dataType.getText( ) ) )
				return getBlankPropertyStatus( cellLabels[1] );
			if ( isBlankProperty( expression.getText( ) ) )
			{
				String funcName = this.getSelectedFuncName( aggrCombo );
				if ( isFunctionCount( funcName ) == false )
					return getBlankPropertyStatus( cellLabels[2] );
			}
			if ( needArguments( ) && isBlankProperty( argument.getText( ) ) )
			{
				return getBlankPropertyStatus( cellLabels[4] );
			}
			return getOKStatus( );
		}

		private final boolean isUniqueColumnName( )
		{
			Iterator iter = computedColumns.iterator( );
			if ( iter != null )
			{
				while ( iter.hasNext( ) )
				{
					ComputedColumnHandle handle = (ComputedColumnHandle) iter.next( );
					if ( getStructure( getStructureOrHandle( ) ) != handle.getStructure( )
							&& handle.getName( ).equals( columnName.getText( ) ) )
					{
						return false;
					}
				}
			}
			Iterator iter2 = ( (DataSetHandle) getContainer( ).getModel( ) ).getPropertyHandle( DataSetHandle.COLUMN_HINTS_PROP ).iterator( );
			while ( iter2.hasNext( ) )
			{
				ColumnHintHandle hint = (ColumnHintHandle) iter2.next( );
				if ( columnName.getText( ).equals( hint.getAlias( ) )
						|| columnName.getText( ).equals( hint.getColumnName( ) ) )
				{
					return false;
				}
			}

			return true;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.birt.report.designer.data.ui.dataset.PropertyHandleInputDialog#rollback()
		 */
		protected void rollback( )
		{
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.birt.report.designer.data.ui.dataset.PropertyHandleInputDialog#getTitle()
		 */
		protected String getTitle( )
		{
			if ( this.getStructureOrHandle( ) instanceof Structure )
			{
				return Messages.getString( "DataSetComputedColumnsPage.toolTipText.New" );
			}
			return Messages.getString( "DataSetComputedColumnsPage.toolTipText.Edit" );//$NON-NLS-1$
		}

	}

}
