/*******************************************************************************
 * Copyright (c) 2004 Actuate Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Actuate Corporation  - initial API and implementation
 *******************************************************************************/

package org.eclipse.birt.report.designer.ui.lib.explorer.action;

import java.io.File;
import java.net.URL;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.birt.report.designer.core.model.SessionHandleAdapter;
import org.eclipse.birt.report.designer.internal.ui.util.UIUtil;
import org.eclipse.birt.report.designer.nls.Messages;
import org.eclipse.birt.report.designer.ui.ReportPlugin;
import org.eclipse.birt.report.designer.ui.lib.explorer.LibraryExplorerTreeViewPage;
import org.eclipse.birt.report.model.api.IResourceLocator;
import org.eclipse.birt.report.model.api.LibraryHandle;
import org.eclipse.birt.report.model.api.ModuleHandle;
import org.eclipse.birt.report.model.api.css.CssStyleSheetHandle;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.IPathEditorInput;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PlatformUI;

/**
 * 
 */

public class DeleteLibraryandCSSAction extends Action
{

	private static final String ACTION_TEXT = Messages.getString( "DeleteLibraryAction.Text" ); //$NON-NLS-1$
	private LibraryExplorerTreeViewPage viewerPage;

	private static final String BUNDLE_PROTOCOL = "bundleresource://"; //$NON-NLS-1$

	public DeleteLibraryandCSSAction( LibraryExplorerTreeViewPage page )
	{
		super( ACTION_TEXT );
		this.viewerPage = page;
	}

	public boolean isEnabled( )
	{
		return getSelectedLibraries( ) != null;
	}

	private List getSelectedLibraries( )
	{
		if ( viewerPage.getTreeViewer( ).getSelection( ) == null
				|| viewerPage.getTreeViewer( ).getSelection( ).isEmpty( ) )
			return null;
		List libraries = new ArrayList( );
		List files = ( (IStructuredSelection) viewerPage.getTreeViewer( )
				.getSelection( ) ).toList( );
		for ( Iterator iter = files.iterator( ); iter.hasNext( ); )
		{
			Object element = (Object) iter.next( );
			if ( element instanceof LibraryHandle )
			{
				LibraryHandle library = (LibraryHandle) element;
				if ( library.getFileName( ).startsWith( BUNDLE_PROTOCOL ) )
					return null;
				libraries.add( library );
			}
			else if ( element instanceof CssStyleSheetHandle )
			{
				CssStyleSheetHandle library = (CssStyleSheetHandle) element;
				if ( library.getFileName( ).startsWith( BUNDLE_PROTOCOL ) )
					return null;
				libraries.add( library );
			}
			else
			{
				return null;
			}
		}
		return libraries.size( ) > 0 ? libraries : null;
	}

	public void run( )
	{
		List libraries = getSelectedLibraries( );

		for ( Iterator iter = libraries.iterator( ); iter.hasNext( ); )
		{
			Object obj = iter.next( );
			if ( !( obj instanceof LibraryHandle || obj instanceof CssStyleSheetHandle ) )
			{
				return;
			}

			try
			{

				File file = null;
				if ( obj instanceof LibraryHandle )
				{
					LibraryHandle node = (LibraryHandle) obj;
					file = new File( new URL( node.getFileName( ) ).getPath( ) );
				}
				else if ( obj instanceof CssStyleSheetHandle )
				{
					CssStyleSheetHandle node = (CssStyleSheetHandle) obj;
					ModuleHandle module = SessionHandleAdapter.getInstance( ).getReportDesignHandle( );
					URL url = module.findResource( node.getFileName( ),IResourceLocator.CASCADING_STYLE_SHEET);					
					file = new File(  url.getFile( ) );
				}

				if ( file == null )
				{
					return;
				}

				if ( file.exists( ) )
				{
					file.delete( );
					String resourceFolder = ReportPlugin.getDefault( )
							.getResourceFolder( );
					String filePath = file.getPath( );
					if ( filePath.startsWith( new File( resourceFolder ).getPath( ) ) )
					{
						// refresh project
						IProject[] projects = ResourcesPlugin.getWorkspace( )
								.getRoot( )
								.getProjects( );
						for ( int i = 0; i < projects.length; i++ )
						{
							if ( projects[i].getLocation( )
									.toFile( )
									.getPath( )
									.equals( new File( resourceFolder ).getPath( ) ) )
							{
								projects[i].refreshLocal( IResource.DEPTH_INFINITE,
										null );
								break;
							}
						}
						// close editor
						IWorkbenchPage pg = PlatformUI.getWorkbench( )
								.getActiveWorkbenchWindow( )
								.getActivePage( );
						IEditorReference[] editors = pg.getEditorReferences( );
						for ( int i = 0; i < editors.length; i++ )
						{
							Object adapter = editors[i].getEditorInput( )
									.getAdapter( IFile.class );
							if ( adapter != null )
							{
								if ( ( (IFile) adapter ).getFullPath( )
										.toFile( )
										.getPath( )
										.equals( filePath ) )
									editors[i].getEditor( false ).dispose( );
							}
							else if ( editors[i].getEditorInput( ) instanceof IPathEditorInput )
							{
								File fileSystemFile = ( (IPathEditorInput) editors[i].getEditorInput( ) ).getPath( )
										.toFile( );
								if ( fileSystemFile.getPath( )
										.equals( filePath ) )
									pg.closeEditor( editors[i].getEditor( false ),
											false );
							}
						}
					}
				}
			}
			catch ( Exception e )
			{
			}

		}
		viewerPage.refreshRoot( );
	}
}
